unit GpCoordinateMapper;

interface

uses w3system, GpW3;

type
  TCoordinateMapping = class // view <-> world mapping
  private
    FView : TIntRect;
    FWorld: TFloatRect;
  public
    function  MapToWorldX(X: integer): float;
    function  MapToWorldY(Y: integer): float;
    procedure OffsetWorldByViewUnits(dX, dY: integer);
    function  ScaleToWorldX(dX: integer): float;
    function  ScaleToWorldY(dY: integer): float;
    procedure SetWorldToViewUnits(rect: TRect);
    function  ViewToWorld(rect: TRect): TFloatRect;
    property View: TIntRect read FView write FView;
    property World: TFloatRect read FWorld write FWorld;
  end;

implementation

function TCoordinateMapping.MapToWorldX(X: integer): float;
begin
  Result := FWorld.Left + ScaleToWorldX(X - FView.Left) ;
end;

function TCoordinateMapping.MapToWorldY(Y: integer): float;
begin
  Result := FWorld.Top + ScaleToWorldY(Y - FView.Top);
end;

procedure TCoordinateMapping.OffsetWorldByViewUnits(dX, dY: integer);
begin
  FWorld.Offset(ScaleToWorldX(dX), ScaleToWorldY(dY));
end;

function TCoordinateMapping.ScaleToWorldX(dX: integer): float;
begin
  Result := dX/ FView.Width * FWorld.Width;
end;

function TCoordinateMapping.ScaleToWorldY(dY: integer): float;
begin
  Result := dY/ FView.Height * FWorld.Height;
end;

procedure TCoordinateMapping.SetWorldToViewUnits(rect: TRect);
begin
  FWorld := ViewToWorld(rect);
end;

function TCoordinateMapping.ViewToWorld(rect: TRect): TFloatRect;
begin
  Result.SetBounds(MapToWorldX(rect.Left), MapToWorldY(rect.Top),
    MapToWorldX(rect.Right), MapToWorldY(rect.Bottom));
end;

end.