unit GpEmbelish;

interface

uses w3system, w3graphics, GpW3;

type
  TEmbelishCanvas = class
  private
    FCanvas_ref: TW3Canvas;
    FArrow:record
      Active: boolean;
      PFrom : TPoint;
      PTo   : TPoint;
      Image : TW3ImageData;
    end;
    FMarquee: record
      Active: boolean;
      Rect  : TRect;
      Left,
      Top,
      Right,
      Bottom: TW3ImageData;
    end;
  protected
    procedure GrabArrow(pFrom, pTo: TPoint);
    procedure GrabMarquee(rect: TRect);
  public
    constructor Create(canvas: TW3Canvas);
    procedure Hide;
    procedure PaintArrow(pFrom, pTo: TPoint);
    procedure PaintMarquee(rect: TRect);
    procedure ShowArrow(pFrom, pTo: TPoint);
    procedure ShowMarquee(rect: TRect);
  end;

implementation

{ TEmbelishCanvas }

constructor TEmbelishCanvas.Create(canvas: TW3Canvas);
begin
  inherited Create;
  FCanvas_ref := canvas;
end;

procedure TEmbelishCanvas.GrabArrow(pFrom, pTo: TPoint);
var
  rect: TRect;
begin
  FArrow.Active := true;
  FArrow.PFrom := pFrom;
  FArrow.PTo := pTo;
  rect := FixOrientation(TRect.Create(pFrom.X, pFrom.Y, pTo.X, pTo.Y));
  FArrow.Image := FCanvas_ref.GetImageData(rect.Left - 1, rect.Top - 1,
    rect.Right - rect.Left + 3, rect.Bottom - rect.Top + 3);
end;

procedure TEmbelishCanvas.GrabMarquee(rect: TRect);
var
  width, height: integer;
begin
  width := rect.Right - rect.Left;
  height := rect.Bottom - rect.Top;
  FMarquee.Active := true;
  FMarquee.Rect := rect;
  FMarquee.Top := FCanvas_ref.GetImageData(rect.Left - 1, rect.Top - 1, width + 2, 2);
  FMarquee.Bottom := FCanvas_ref.GetImageData(rect.Left - 1, rect.Bottom - 1, width + 2, 2);
  FMarquee.Left := FCanvas_ref.GetImageData(rect.Left - 1, rect.Top - 1, 2, height + 2);
  FMarquee.Right := FCanvas_ref.GetImageData(rect.Right - 1, rect.Top - 1, 2, height);
end;

procedure TEmbelishCanvas.Hide;
var
  rect: TRect;
begin
  if FArrow.Active then begin
    rect := FixOrientation(TRect.Create(FArrow.PFrom.X, FArrow.PFrom.Y, FArrow.PTo.X, FArrow.PTo.Y));
    FCanvas_ref.PutImageData(FArrow.Image, rect.Left - 1, rect.Top - 1);
    FArrow.Active := false;
  end;
  if FMarquee.Active then begin
    FCanvas_ref.PutImageData(FMarquee.Top, FMarquee.Rect.Left - 1, FMarquee.Rect.Top - 1);
    FCanvas_ref.PutImageData(FMarquee.Bottom, FMarquee.Rect.Left - 1, FMarquee.Rect.Bottom - 1);
    FCanvas_ref.PutImageData(FMarquee.Left, FMarquee.Rect.Left - 1, FMarquee.Rect.Top - 1);
    FCanvas_ref.PutImageData(FMarquee.Right, FMarquee.Rect.Right - 1, FMarquee.Rect.Top - 1);
    FMarquee.Active := false;
  end;
end;

procedure TEmbelishCanvas.PaintArrow(pFrom, pTo: TPoint);
begin
  FCanvas_ref.StrokeStyle := '#FFFFFF';
  FCanvas_ref.LineWidth := 1;
  FCanvas_ref.BeginPath;
  FCanvas_ref.LineF(pFrom.X, pFrom.Y, pTo.X, pTo.Y);
  FCanvas_ref.Stroke;
  FCanvas_ref.ClosePath;
end;

procedure TEmbelishCanvas.PaintMarquee(rect: TRect);
begin
  FCanvas_ref.StrokeStyle := '#FFFFFF';
  FCanvas_ref.LineWidth := 1;
  FCanvas_ref.StrokeRectF(rect.Left - 0.5, rect.Top - 0.5, rect.Right - rect.Left + 1, rect.Bottom - rect.Top + 1);
end;

procedure TEmbelishCanvas.ShowArrow(pFrom, pTo: TPoint);
begin
  Hide;
  GrabArrow(pFrom, pTo);
  PaintArrow(pFrom, pTo);
end;

procedure TEmbelishCanvas.ShowMarquee(rect: TRect);
begin
  Hide;
  if (rect.Right = rect.Left) or (rect.Top = rect.Bottom) then
    Exit;
  GrabMarquee(rect);
  PaintMarquee(rect);
end;

end.
