unit GpW3;

interface

uses w3system, w3graphics, w3touch;

{ time }

const HoursPerDay   = 24;
const MinsPerHour   = 60;
const SecsPerMin    = 60;
const MSecsPerSec   = 1000;
const MinsPerDay    = HoursPerDay * MinsPerHour;
const SecsPerDay    = MinsPerDay * SecsPerMin;
const MSecsPerDay   = SecsPerDay * MSecsPerSec;

function HasElapsedMs(start: float; interval_ms: integer): boolean;

{ points and rectangles }

function Point(X, Y: integer): TPoint;

type
  TIntRect = record
    Left,
    Top,
    Right,
    Bottom,
    Width,
    Height: integer;
    function  AsString: string;
    function  Midpoint: TPoint;
    procedure Offset(dX, dY: integer);
    procedure SetBounds(aLeft, aTop, aRight, aBottom: integer);
  end;

function IntRect(left, top, right, bottom: integer): TIntRect;
function IntRectEx(left, top, width, height: integer): TIntRect;

type
  TFloatPoint = record
    X,
    Y: float;
    function  AsString: string;
  end;

function FloatPoint(X, Y: float): TFloatPoint;

type
  TFloatRect = record
    Left,
    Top,
    Right,
    Bottom,
    Width,
    Height: float;
    function  AsString: string;
    function  Midpoint: TFloatPoint;
    procedure Offset(dX, dY: float);
    procedure SetBounds(aLeft, aTop, aRight, aBottom: float);
  end;

function FloatRect(left, top, right, bottom: float): TFloatRect;
function FloatRectEx(left, top, width, height: float): TFloatRect;

function FixOrientation(rect: TRect): TRect;

{ misc }

function MakeColor(R, G, B, A: integer): TW3RGBA;
function TouchToStr(touchList: TW3TouchList): string;

implementation

{ time }

function HasElapsedMs(start: float; interval_ms: integer): boolean;
begin
  Result := ((Now - start) * MSecsPerDay) >= interval_ms;
end;

{ points and rectangles }

function Point(X, Y: integer): TPoint;
begin
  Result.X := X;
  Result.Y := Y;
end;

function FloatPoint(X, Y: float): TFloatPoint;
begin
  Result.X := X;
  Result.Y := Y;
end;

function TFloatPoint.AsString: string;
begin
  Result := Format('%f,%f', [X, Y]);
end;

function IntRect(left, top, right, bottom: integer): TIntRect;
begin
  Result.SetBounds(left, top, right, bottom);
end;

function IntRectEx(left, top, width, height: integer): TIntRect;
begin
  Result := IntRect(left, top, left+width, top+height);
end;

function FloatRect(left, top, right, bottom: float): TFloatRect;
begin
  Result.SetBounds(left, top, right, bottom);
end;

function FloatRectEx(left, top, width, height: float): TFloatRect;
begin
  Result := FloatRect(left, top, left+width, top+height);
end;

function FixOrientation(rect: TRect): TRect;
begin
  Result := rect;
  if Result.Left > Result.Right then begin
    Result.Left := rect.Right;
    Result.Right := rect.Left;
  end;
  if Result.Top > Result.Bottom then begin
    Result.Top := rect.Bottom;
    Result.Bottom := rect.Top;
  end;
end;

{ TIntRect }

function TIntRect.AsString: string;
begin
  Result := Format('%d,%d - %d,%d', [Left, Top, Right, Bottom]);
end;

function TIntRect.Midpoint: TPoint;
begin
  Result := Point((Left + Width) div 2, (Top + Height) div 2);
end;

procedure TIntRect.Offset(dX, dY: integer);
begin
  Left := Left + dX;
  Top := Top + dY;
  Right := Right + dX;
  Bottom := Bottom + dY;
end;

procedure TIntRect.SetBounds(aLeft, aTop, aRight, aBottom: integer);
begin
  Left := aLeft;
  Top := aTop;
  Right := aRight;
  Bottom := aBottom;
  Width := Right - Left;
  Height := Bottom - Top;
end;

{ TFloatRect }

function TFloatRect.AsString: string;
begin
  Result := Format('%f,%f - %f,%f', [Left, Top, Right, Bottom]);
end;

function TFloatRect.Midpoint: TFloatPoint;
begin
  Result := FloatPoint(Left + Width/2, Top + Height/2);
end;

procedure TFloatRect.Offset(dX, dY: float);
begin
  Left := Left + dX;
  Top := Top + dY;
  Right := Right + dX;
  Bottom := Bottom + dY;
end;

procedure TFloatRect.SetBounds(aLeft, aTop, aRight, aBottom: float);
begin
  Left := aLeft;
  Top := aTop;
  Right := aRight;
  Bottom := aBottom;
  Width := Right - Left;
  Height := Bottom - Top;
end;

{ misc }

function MakeColor(R, G, B, A: integer): TW3RGBA;
begin
  Result.R := R;
  Result.G := G;
  Result.B := B;
  Result.A := A;
end;

function TouchToStr(touchList: TW3TouchList): string;
var
  i: integer;
begin
  Result := '';
  for i := 0 to touchList.Count - 1 do begin
    if Result <> '' then
      Result := Result + '; ';
    Result := Result + '#' + IntToStr(i) + ':' +
      Format('%s:%d,%d/%d,%d/%d,%d', [touchList[i].Identifier,
        touchList[i].ScreenX, touchList[i].ScreenY,
        touchList[i].ClientX, touchList[i].ClientY,
        touchList[i].PageX,   touchList[i].PageY]);
  end;
end;

end.
